﻿---------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Assessment")
local SML = AceLibrary("SharedMedia-1.0")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["History"] = true,
	["Number of fights to keep detailed information for. Use 0 to keep all."] = true,
	["Pause"] = true,
	["Pause gathering combat data."] = true,

	["Reset"] = true,
	["Resets fight data. Does not reset saved fight data."] = true,
	["Reset All"] = true,
	["Reset all fight data, including saved fight data."] = true,

	["Data Options"] = true,
	["Data options."] = true,

    ["Enumerate Data"] = true,
    ["Enumerate Parameter List"] = true,
    ["FilterAll"] = "Show All",
	["FilterNonZero"] = "Show Non-Zero Values",
	
	["All fight data has been reset"] = true,
	["Fight data has been reset"] = true,

	["Player"] = true,
	["Player Color"] = true,
	["Select the color to use for Player colortype."] = true,
	
	["Gather OOC"] = true,
	["Enable gathering of data while not in combat. Defaults to false."] = true,

	["Enter a name to save the fight data."] = true,

	["Erase Fight Data"] = true,
	["Erase specific fight data."] = true,
	["Erase %s"] = true,
	["Click to permanently delete the fight data named \"%s\""] = true,
} end)

-----------------------------------------------------------------------------

Assessment = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0")
Assessment.Debug = (UnitName("player") == "Roartindon") and function(_, text) ChatFrame3:AddMessage(text) end or function() end
Assessment.version = "r"..("$Revision: 42728 $"):match("%d+")
Assessment:RegisterDB("AssessmentDB", "AssessmentDBPerCharacter")
Assessment:RegisterDefaults('char', { fightData = { } } )
Assessment:RegisterDefaults('profile', { 
	history				= 3,
	pause				= false,
	gatherOutOfCombat	= false,
	playerColor			= { 1, 0, 1 },
} )

-----------------------------------------------------------------------------

local combatTime
local fightData
local profile

-----------------------------------------------------------------------------

local eraseFightDataArgs = { }
local eraseFightDataDirty = true
local eraseFightDataMetaTable = 
{ 
	__index = function(self, key)
				if key == "args" then
					if eraseFightDataDirty then
						Assessment:UpdateEraseFightData()
					end
					return eraseFightDataArgs
				end
			  end
}

Assessment.Options = 
{
	handler = Assessment,
	type = 'group',
	args = 
	{
		reset =
		{
			type = "execute",
			name = L["Reset"],
			desc = L["Resets fight data. Does not reset saved fight data."],
			func = function() Assessment:ResetUnnamedFightData() end,
			order = 1,
		},
		resetAll =
		{
			type = "execute",
			name = L["Reset All"],
			desc = L["Reset all fight data, including saved fight data."],
			func = function() Assessment:ResetFightData() end,
			order = 2,
		},
		eraseFightData = setmetatable(
		{
			type = "group",
			name = L["Erase Fight Data"],
			desc = L["Erase specific fight data."],
			disabled = function() return not next(eraseFightDataArgs) end,
			order = 3
		}, eraseFightDataMetaTable),
		history =
		{
			type = "range",
			name = L["History"],
			desc = L["Number of fights to keep detailed information for. Use 0 to keep all."],
			max = 20,
			min = 0,
			step = 1,
			get = function() return profile.history end,
			set = function(value) profile.history = value end,
			order = 10,
		},
		pause =
		{
			type = "toggle",
			name = L["Pause"],
			desc = L["Pause gathering combat data."],
			get = function() return profile.pause end,
			set = function(value) profile.pause  = value end,
			order = 10,
		},
		dataSpacer = 
		{
			type = 'header',
			order = 90,
		},
		data =
		{
			type = 'group',
			name = L["Data Options"],
			desc = L["Data options."],
			order = 100,
			args = 
			{
				playerColor =
				{
					type = "color",
					name = L["Player Color"],
					desc = L["Select the color to use for Player colortype."],
					get = function() return unpack(profile.playerColor) end,
					set = function(r, g, b) profile.playerColor = { r, g, b } Assessment:RedrawAll() end,
				},
				gatherOutOfCombat =
				{
					type = "toggle",
					name = L["Gather OOC"],
					desc = L["Enable gathering of data while not in combat. Defaults to false."],
					get = function() return profile.gatherOutOfCombat end,
					set = function(v) profile.gatherOutOfCombat = v end,
				},
			},
		}
	},
}

local function EraseFightDataMethod(name) Assessment:EraseFightData(name) end
function Assessment:UpdateEraseFightData()
	if not fightData then return end
	self:ResetTable(eraseFightDataArgs, true)
	eraseFightDataDirty = false
	
	for k in pairs(fightData) do
		if type(k) == "string" then
			local args = self:NewTable()
			args.type = "execute"
			args.name = (L["Erase %s"]):format(k)
			args.desc = (L["Click to permanently delete the fight data named \"%s\""]):format(k)
			args.func = EraseFightDataMethod
			args.passValue = k
			eraseFightDataArgs[k] = args
		end
	end
end

-----------------------------------------------------------------------------

Assessment:RegisterChatCommand("/as", "/assessment", Assessment.Options)

-----------------------------------------------------------------------------

Assessment.ColorType			= { }
Assessment.DefaultDisplaySets	= { }
Assessment.DefaultTags			= { }
Assessment.FilterType			= { }
Assessment.ListType				= { }

-----------------------------------------------------------------------------

Assessment.EmptyMethod = function() end
local emptyMethod = Assessment.EmptyMethod

-----------------------------------------------------------------------------

local playerName = UnitName("player")
Assessment.ColorType.Player = 
{
	method = function(fightData, data)
				if data.key ~= playerName then return end
				return profile.playerColor
			 end,
	description = L["Player"]
}

Assessment.FilterType.All =
{
	tag			= "1",
	description	= L["FilterAll"],
}

Assessment.FilterType.NonZero =
{
	tag			= "BarValue ~= 0",
	description	= L["FilterNonZero"],
}

Assessment.ListType.EnumerateTable =
{
	method = function(data, ...)
				for i=1,select('#',...) do
					if type(data) ~= "table" then return emptyMethod end
					local lookup = select(i, ...)
					data = data[lookup]
				end
				if type(data) ~= "table" then return emptyMethod end
				return pairs(data)
			 end,

	data = function(listTypeData)
				return Assessment.ListType.EnumerateTable.updateData[listTypeData]
		   end,
	
	description = L["Enumerate Data"],
	
	updateData = { }
}

-----------------------------------------------------------------------------

local IterateParameterList
do
	local index
	
	local function ParameterListIterator(state)
		index = index + 1
		if index > #state then
			Assessment:ReleaseTable(state)
		else
			return state[index]
		end
	end

	function IterateParameterList(state)
		if #state == 0 then
			Assessment:ReleaseTable(state)
			return emptyMethod
		else
			index = 0
			return ParameterListIterator, state, nil
		end
	end
end

Assessment.ListType.EnumerateParameterList =
{
	method = function(data, ...)
				local state = Assessment:NewTable()
				for i=1,select('#',...) do
					state[i] = select(i, ...)
				end
				return IterateParameterList(state)
			 end,

	description = L["Enumerate Parameter List"],
}

-----------------------------------------------------------------------------

function Assessment:OnInitialize()
	profile = self.db.profile
	combatTime = self:GetModule("CombatTime")
	fightData = self.db.char.fightData

	if not fightData[0] then
		fightData[0] = self:NewFightDataTable()
	end

	-- Install a GetFightData that actuall does something
	function Assessment:GetFightData(index)
		return fightData[index]
	end
end

function Assessment:OnEnable()
end

function Assessment:OnDisable()
end

-----------------------------------------------------------------------------

function Assessment:ResetFightData()
	eraseFightDataDirty = true

	for k,v in pairs(fightData) do
		fightData[k] = self:ReleaseTable(fightData[k], true)
	end
	fightData[0] = self:NewFightDataTable()
	self:TriggerEvent("Assessment:ResetFightData")
	
	self:Print(L["All fight data has been reset"])
end

function Assessment:ResetUnnamedFightData()
	for k,v in pairs(fightData) do
		if type(k) ~= "string" then
			fightData[k] = self:ReleaseTable(fightData[k], true)
		end
	end
	fightData[0] = self:NewFightDataTable()
	self:TriggerEvent("Assessment:ResetFightData")
	
	self:Print(L["Fight data has been reset"])
end

function Assessment:NewFightDataTable()
	local data = self:NewTable()
	data.dataUpdateTimes = self:NewTable()
	self:TriggerEvent("Assessment:NewFightDataTable", data)
	return data
end

function Assessment:CopyFightData(targetIndex, sourceIndex)
	eraseFightDataDirty = true
	if fightData[targetIndex] then
		fightData[targetIndex] = self:ReleaseTable(fightData[targetIndex], true)
	end
	
	if not fightData[sourceIndex] then return end
	
	local newFightData = self:CopyTable(fightData[sourceIndex])
	fightData[targetIndex] = newFightData
	
	self:TriggerEvent("Assessment:CopyFightData", targetIndex, newFightData)
end

function Assessment:EraseFightData(key)
	if not fightData[key] then return end
	eraseFightDataDirty = true
	fightData[key] = self:ReleaseTable(fightData[key], true)
	self:TriggerEvent("Assessment:ResetFightData")
end

-----------------------------------------------------------------------------

function Assessment:StartNewFight()
	if profile.pause then return end

	table.insert(fightData, 1, self:NewFightDataTable())
	
	if profile.history ~= 0 then
		for i=#fightData, profile.history+1, -1 do
			fightData[i] = self:ReleaseTable(fightData[i], true)
		end
	end
end

-----------------------------------------------------------------------------

local dataTablePool = setmetatable({ }, { __mode="k" })

function Assessment:NewTable()
	local recycledTable = next(dataTablePool)
	if recycledTable then
		dataTablePool[recycledTable] = nil
		return recycledTable
	else
		return { }
	end
end

function Assessment:ReleaseTable(t, recursive)
	for k,v in pairs(t) do
		if recursive and type(v) == "table" then
			self:ReleaseTable(v, true)
		end
		t[k] = nil
	end
	dataTablePool[t] = true
end

function Assessment:ResetTable(t, recursive)
	for k,v in pairs(t) do
		if recursive and type(v) == "table" then
			self:ReleaseTable(v, true)
		end
		t[k] = nil
	end
end

function Assessment:CopyTable(source)
	local result = self:NewTable()

	for k,v in pairs(source) do
		if type(v) == "table" then
			result[k] = self:CopyTable(v)
		else
			result[k] = v
		end
	end
	
	return result
end

-----------------------------------------------------------------------------

-- Empty, replaced in OnInitialize
-- Some modules end up calling GetFightData, so it needs to be present, but
-- return nil until we're ready.
function Assessment:GetFightData(index)
end

-----------------------------------------------------------------------------

-- These iterators are not to be called in a nested manner.
local iterateIndex = -1
local function DataTableIterator()
	if iterateIndex < 1 then
		iterateIndex = iterateIndex+1
		return fightData[iterateIndex]
	end
end

local function ForceIterateUpdateDataTables()
	iterateIndex = -1
	return DataTableIterator
end

local function IterateUpdateDataTables()
	if profile.pause or (not profile.gatherOutOfCombat and not combatTime:IsInCombat()) then
		return emptyMethod
	else 
		return ForceIterateUpdateDataTables()
	end
end

-----------------------------------------------------------------------------

function Assessment:RedrawAll()
	self:TriggerEvent("Assessment:RedrawAll")
end

-----------------------------------------------------------------------------

Assessment.modulePrototype.IterateUpdateDataTables = IterateUpdateDataTables
Assessment.modulePrototype.ForceIterateUpdateDataTables = ForceIterateUpdateDataTables
Assessment.modulePrototype.NewTable = Assessment.NewTable
Assessment.modulePrototype.ReleaseTable = Assessment.ReleaseTable
Assessment.modulePrototype.ResetTable = Assessment.ResetTable
Assessment.modulePrototype.CopyTable = Assessment.CopyTable

function Assessment.modulePrototype.IterateFightData()
	return pairs(fightData)
end

function Assessment.modulePrototype.GetOptionValue(data, lookup)
	local v
	while lookup do
		v, lookup = ("."):split(lookup, 2)
		data = data[v]
	end
	return data
end

function Assessment.modulePrototype.SetOptionValue(data, lookup, value)
	local v
	while true do
		v, lookup = ("."):split(lookup, 2)		
		if lookup then
			data = data[v]
		else
			data[v] = value
			return
		end
	end
end

function Assessment.modulePrototype:CopyOptionsTemplate(source, i)
	local result = Assessment:NewTable()

	for k,v in pairs(source) do
		if k == "passValue" then
			local passValueTable = self:NewTable()
			passValueTable[1] = i
			if type(v) == "table" then
				passValueTable[2] = self:CopyTable(v)
			else
				passValueTable[2] = v
			end
			result[k] = passValueTable
		elseif type(v) == "table" then
			result[k] = self:CopyOptionsTemplate(v, i)
		else
			result[k] = v
		end
	end
	
	return result
end

function Assessment.modulePrototype:Debug(message)
	Assessment:Debug(message)
end

function Assessment.modulePrototype:CancelEvent(eventName)
	if self:IsEventRegistered(eventName) then
		self:UnregisterEvent(eventName)
	end
end

-----------------------------------------------------------------------------

StaticPopupDialogs["ASSESSMENT_SAVE_FIGHT_DATA"] = 
{
	text = L["Enter a name to save the fight data."],
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	OnAccept = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		if text ~= "" then
			Assessment:CopyFightData(text, StaticPopupDialogs["ASSESSMENT_SAVE_FIGHT_DATA"].data)
		end
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetText("")
		getglobal(this:GetName().."EditBox"):SetFocus()
	end,
	EditBoxOnEnterPressed = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		if text ~= "" then
			Assessment:CopyFightData(text, StaticPopupDialogs["ASSESSMENT_SAVE_FIGHT_DATA"].data)
		end
		this:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
}

-----------------------------------------------------------------------------
