-----------------------------------------------------------------------------

local Roster = AceLibrary("Roster-2.1")
local L = AceLibrary("AceLocale-2.2"):new("AssessmentUnitData")
L:RegisterTranslations("enUS", function() return {
    ["Group Health"] = true,
    ["Group Mana"] = true,
} end)

-----------------------------------------------------------------------------

Assessment.TagData.Health =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitHealth(unitid)
			 end,
	events			= "UNIT_HEALTH",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.MaximumHealth =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitHealthMax(unitid)
			 end,
	events			= "UNIT_HEALTH",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.HealthFraction =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitHealth(unitid) / UnitHealthMax(unitid)
			 end,
	events			= "UNIT_HEALTH",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.Power =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitMana(unitid)
			 end,
	events			= "UNIT_MANA",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.PowerFraction =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitMana(unitid) / UnitManaMax(unitid)
			 end,
	events			= "UNIT_MANA",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.PowerType =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitPowerType(unitid)
			 end,
	events			= "UNIT_DISPLAYPOWER",
	numeric			= true,
	guaranteeResult = true
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.GroupHealth =
{
	titleTag		= "Group Health",
	listType		= "CurrentGroupMembers", 
	barValueTag		= "HealthFraction",
	filterTag		= "1",
	fightIndex		= 0,
	text			= { { }, { tag= "[(BarValue*100):Percent]" } },
	description		= L["Group Health"],
	colorTypeList	= { "Class", count = 1 },
	favorite		= false,
}

Assessment.DefaultDisplaySets.GroupMana =
{
	titleTag		= "Group Mana",
	listType		= "CurrentGroupMembers", 
	barValueTag		= "PowerFraction",
	filterTag		= "PowerType == 0",
	fightIndex		= 0,
	text			= { { }, { tag= "[(BarValue*100):Percent]" } },
	description		= L["Group Mana"],
	colorTypeList	= { "Class", count = 1 },
	favorite		= false,
}

-----------------------------------------------------------------------------
